
/*
 * master.cpp - Functions related to the master process (NQueens Parallel Version)
 * (c) Paulo Marques, pmarques@dei.uc.pt
 * April 03, 2003
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "nqueens.h"
#include "mpi.h"

// Wait for any pending results. The functions returns the number
// of solutions found meanwhile
int wait_remaining_results()
{
  int n_solutions = 0;

  // Wait for the remaining results, sending a quit whenever a new result arives
  while (n_workers > 0)
  {
    // Receive message from a worker
    worker_msg msg;
    MPI_Recv(&msg, sizeof(msg), MPI_BYTE, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, MPI_STATUS_IGNORE);

    n_solutions+= msg.solutions_found;

    job byebye;
    byebye.type = QUIT;
    MPI_Send(&byebye, sizeof(byebye), MPI_BYTE, msg.origin, 0, MPI_COMM_WORLD);
    --n_workers;
  }

  return n_solutions;
}

// Sends a job to a worker. The first <granularity> columns are a valid
// solution (until that point), which corresponds to the job being
// investigated. The function returns the number of solutions found
// by different workers while in this routine
int send_job_worker(int board[])
{
  int n_solutions = 0;      // The number of solutions found meanwhile
  job to_do;                // The job to do

  // Build the job
  to_do.type = DO_WORK;
  for (int i=0; i<granularity; i++)
    to_do.board[i] = board[i];

  // Receive message from a worker
  worker_msg msg;
  MPI_Recv(&msg, sizeof(msg), MPI_BYTE, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
  n_solutions = msg.solutions_found;

  // Send the a new job to the worker
  MPI_Send(&to_do, sizeof(to_do), MPI_BYTE, msg.origin, 0, MPI_COMM_WORLD);

  return n_solutions;
}

// Place queens in all possible ways, backtracking if necessary, in
// column <column>. <board> is used as current board.
// 
// The first <granularity> columns are used by the master. After
// that, the master sends that prefix, which constitutes a job,
// to the first available worker
int master_place_queen(int column, int board[])
{
  // Total solutions for this level
  int n_solutions = 0;

  // Try to place a queen in each line of the <column> column
  for (int i=0; i<size; i++)
  {
    // Place the queen on the correct line of <column>
    board[column] = i;
    
    // Check if this board is still a solution
    bool is_sol = true;
    for (int j=column-1; j>=0; j--)
    {
      if ((board[column] == board[j])               ||
          (board[column] == board[j] - (column-j))  ||
          (board[column] == board[j] + (column-j)))
      {
        is_sol = false;
        break;
      }
    }
    if (is_sol)     
    {
      // If it is a solution (well, the first granularity columns)...

      if (column == granularity-1)
      {
        // If we are at the last level, this is a job for sending to a worker
        n_solutions+= send_job_worker(board);
      }
      else
      {
        // Not in the last level, try to place queens in the 
        // next one using the current board
        n_solutions+= master_place_queen(column+1, board);
      }
    }
  }

  return n_solutions;
}

// The master process
int master()
{
  int* board = (int*) calloc(size, sizeof(int));

  // Start out calculating the solutions by placing queens
  // on the first column of the board
  int n_solutions = master_place_queen(0, board);

  // Wait out for pending solutions (there may be workers still calculating)
  n_solutions+= wait_remaining_results();
  
  return n_solutions;
}
