
/*
 * nqueens.cpp - Main program (NQueens Parallel Version)
 * (c) Paulo Marques, pmarques@dei.uc.pt
 * April 03, 2003
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "nqueens.h"
#include "mpi.h"

/******************************/

// Every single process knows this information

int id;                                     // The id of this process
int n_workers;                              // Number of workers running
int size        = DEFAULT_SIZE;             // Size of the problem
int granularity = DEFAULT_GRANULARITY;      // Granularity of the jobs

/******************************/

int main(int argc, char* argv[])
{  
  time_t now = 0;  
  time_t done = 0;                            // To time the computation
  int n_solutions = 0;                        // Number of solutions found

  // Init MPI
  MPI_Init(&argc, &argv);

  // Set the size of the world
  MPI_Comm_rank(MPI_COMM_WORLD, &id);
  MPI_Comm_size(MPI_COMM_WORLD, &n_workers);
  --n_workers;                                // Since the master is not a worker

  if (id == MASTER)
  {
    // The master must parse the arguments, finding out the size of the problem
    // and the granuarity of the jobs, and broadcast this information to the
    // workers

    if (argc == 2)
      size = atoi(argv[1]);
    else if (argc == 3)
    {
      size        = atoi(argv[1]);
      granularity = atoi(argv[2]);
    }
  }

  // The master must broadcast the size of the problem and granularity to all workers
  MPI_Bcast(&size, 1, MPI_INT, MASTER, MPI_COMM_WORLD);
  MPI_Bcast(&granularity, 1, MPI_INT, MASTER, MPI_COMM_WORLD);

  // Sanity checks
  if ((size > MAX_SIZE) || (size < 2) || (granularity >= size-1))
  {
    if (id == MASTER)
      printf("Invalid parameters for the problem! (size=%d, granularity=%d)\n", size, granularity);
    MPI_Finalize();
    return 0;
  }
  if (n_workers <= 0)
  {
    if (id == MASTER)
      printf("Must have at least one worker!\n");
    MPI_Finalize();
    return 0;
  }
 

  /*********** MAIN PART ***********/

  if (id == MASTER)
    {
      printf("MASTER running, %d Workers\n", n_workers);
      printf("Problem size=%d, granularity=%d\n", size, granularity);
      printf("\n");
      now = time(NULL);
    }

  // Let there be a master and several workers
  // The master will return the number of solutions
  if (id == MASTER)
    n_solutions = master();
  else
    worker();

  if (id == MASTER)
    {
      done = time(NULL);
      printf("\n");
      printf("Queens %2d: total solutions: %d\n", size, n_solutions);
      printf("It took %ld sec\n", (long)(done-now));
    }

  // All done
  MPI_Finalize();

  return 0;
}
