
/*
 * worker.cpp - Functions related to the worker processes (NQueens Parallel Version)
 * (c) Paulo Marques, pmarques@dei.uc.pt
 * April 03, 2003
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "nqueens.h"
#include "mpi.h"

// Works out the solutions by placing queens in column <column>. It assumes
// that previous columns contain a valid configuration. Any solutions found
// is sent back to the master.
int worker_place_queen(int column, int board[])
{
  int n_solutions = 0;

  // Try to place a queen in each line of the <column> column
  for (int i=0; i<size; i++)
  {
    // Place the queen
    board[column] = i;
    
    // Check if this board is still a solution
    bool is_sol = true;
    for (int j=column-1; j>=0; j--)
    {
      if ((board[column] == board[j])               ||
          (board[column] == board[j] - (column-j))  ||
          (board[column] == board[j] + (column-j)))
      {
        is_sol = false;
        break;
      }
    }
    if (is_sol)
    {
      // The positioning was valid

      if (column == size-1)
      {
        // If we are at the last level, this is a solution, 
        // count it
        ++n_solutions;        
      }
      else
      {
        // Not in the last level, try to place queens in the 
        // next one using the current board
        n_solutions+= worker_place_queen(column+1, board);
      }
    }
  }

  return n_solutions;
}

// Main worker routine. Starts by asking a job and, while an order to quit
// is not received, it processes the job, sending back any solutions found,
// and asks for more work to do.
void worker()
{
  int n_solutions;

	// There is a default message named ask_job which allows a worker to request a job
  // reporting the number of solutions found in the last iteration
	worker_msg ask_job;
  ask_job.origin          = id;
  ask_job.solutions_found = 0;

	// Request initial job
	MPI_Send(&ask_job, sizeof(ask_job), MPI_BYTE, MASTER, 0, MPI_COMM_WORLD);

	while (true)
	{
		// Wait for a job or a quit message
		job work_to_do;
		MPI_Recv(&work_to_do, sizeof(work_to_do), MPI_BYTE, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, MPI_STATUS_IGNORE);

		if (work_to_do.type == QUIT)
			break;

    // We are not quiting, there's a job to process
    n_solutions = worker_place_queen(granularity, work_to_do.board);
    
    // Ask for more work
    ask_job.solutions_found = n_solutions;
    MPI_Send(&ask_job, sizeof(ask_job), MPI_BYTE, MASTER, 0, MPI_COMM_WORLD);
  }
}


