package com.trcinc.common;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.EJBHome;
import javax.ejb.EJBLocalHome;
import javax.ejb.EJBLocalObject;
import javax.ejb.EJBObject;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.axis.MessageContext;
import org.apache.axis.session.Session;

import com.trcinc.security.SecureSession;

/**
 * 
 * Abstract base class provides security method level authorization and 
 * creationg caching of EJB local objects.
 * 
 */
abstract public class AbstractWebService
{
	public static final String SESSION_KEY_SECURE_SESSION = "security.SecureSession";

	/**
	 * Constructor for AbstractWebService.
	 */
	public AbstractWebService()
	{
		super();
	}

	/**
	 * Returns the SOAP session.
	 * 
	 * @return Session
	 */
	public static Session getSession()
	{
		return MessageContext.getCurrentContext().getSession();
	}
	
	/**
	 * Returns the SecureSession object stored in the SOAP session.
	 * 
	 * @return SecureSession
	 */

	public SecureSession getSecureSession()
	{
		return (SecureSession) getSession().get(SESSION_KEY_SECURE_SESSION);
	}
	
	/**
	 * Creates a local EJB object for a business service and cahces it on the SOAP
	 * session.
	 * 
	 * @param ejbLocalHomeJndiName
	 * @param ejbLocalHomeClass
	 * @param ejbLocalSessionName
	 * @return EJBLocalObject
	 * @throws Exception
	 */
	public EJBLocalObject getLocalService(
		String ejbLocalHomeJndiName,
		Class ejbLocalHomeClass,
		String ejbLocalSessionName)
		throws Exception
	{
		Session session = getSession();
		EJBLocalObject service =
			(EJBLocalObject) session.get(ejbLocalSessionName);
		if (service == null)
		{
			try
			{
				InitialContext jndiContext = new InitialContext();
				EJBLocalHome serviceHome = (EJBLocalHome) jndiContext.lookup(ejbLocalHomeJndiName);
				
				service = createLocalEjb (serviceHome);
				
				MessageContext.getCurrentContext().setMaintainSession(true);
				
				session.set(ejbLocalSessionName, service);
			}
			catch (NamingException e)
			{
				throw new RemoteException(
					"Could not locate EJB using JNDI",
					e);
			}
			catch (RemoteException e)
			{
				throw new RemoteException(
					"Could not communicate with EJB home interface",
					e);
			}
			catch (CreateException e)
			{
				throw new RemoteException(
					"Could not create EJB using home interface",
					e);
			}
		}
		return service;
	}
	
	/**
	 * Peforms the method level authorization.
	 * 
	 * In this simple implementation verification is done to
	 * make sure user is logged on to the security service.
	 * 
	 * @param serviceName
	 * @param methodName
	 * @throws SecurityException
	 */
	
	public void authorizeMethodAccess (String serviceName, String methodName) throws SecurityException 
	{
		SecureSession secureSession = getSecureSession ();
		
		if (secureSession == null)
		{
			//
			// Security service did not set the SecureSession.
			//
			throw new SecurityException ("User is not logged on to the system.");
		}
		//
		// Additional security check based on the serviceName and methodName can be performed here.
		//
		return;
	}		

	/**
	 * Create the appropriate EJB local object.
	 * 
	 * @param home
	 * @return EJBLocalObject
	 * @throws CreateException
	 * @throws RemoteException
	 */	
	
	public abstract EJBLocalObject createLocalEjb( EJBLocalHome home ) throws CreateException, RemoteException;
	

}
