package com.trcinc.finance;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.naming.InitialContext;
import javax.sql.DataSource;

import com.trcinc.common.FinanceAppException;

/**
 * 
 * Implements FinancePersistence based on JBoss default data store.
 * 
 * In this simple implementation all persistence is done inside of
 * class this class. But this could also delegate to an Entity Bean.
 * 
 * For simplicity, public methods test for the existence of the stock table and
 * creates and initializes it if is not present.
 * 
 * @author sgovind
 *
 */
public class DefaultPersistence implements FinancePersistence
{


	private String tableName 			= "StockQuote";
	
	private String dropTableSql	    = "DROP TABLE " + tableName;
	
	
	private String tableExistsSql		= "SELECT * FROM StockQuote";
	
    private String createTableSql      = "CREATE TABLE StockQuote (Ticker varchar(20), Price  double)";
    
	private String insertSql           = "INSERT INTO StockQuote (Ticker, Price) VALUES (?, ?)";
	                     
	private String selectSql           = "SELECT Price FROM StockQuote WHERE Ticker=?";                     

	/**
	 * Constructor for FinancePersistence.
	 */
	public DefaultPersistence()
	{
		super();
	}
	

	/**
	 * Adds a ticker/price to the persistence store.
	 * 
	 * @param ticker
	 * @param price
	 * @throws FinanceAppException
	 */
	public void addStockQuote (String ticker, double price) throws FinanceAppException
	{
		try
		{
			Connection conn = getConnection ();
			
			// setup database if needed
			checkAndPrepareDB (conn);
			
			addStockQuote (conn, ticker, price);
		}
		catch (SQLException e)
		{
			throw new FinanceAppException ("addStockQuote failed for ticker (" + ticker + ")", e);
		}
	}
	
	/**
	 * Returns the stock quote for the ticker.
	 * 
	 * @param ticker
	 * @return double
	 * @throws InvalidTickerException
	 * @throws FinanceAppException
	 */

	public double getStockQuote (String ticker) throws InvalidTickerException, FinanceAppException
	{
		try
		{
			Connection conn = getConnection ();
			
			// setup database if needed
			checkAndPrepareDB (conn);
			
			return getStockQuote (conn, ticker);
		}
		catch (SQLException e)
		{
			throw new FinanceAppException ("getStockQuote failed for ticker (" + ticker + ")", e);
		}
	}
	
	/**
	 * Returns a JDBC connection from the JBoss default data source.
	 * 
	 * @return Connection
	 * @throws FinanceAppException
	 */
	private Connection getConnection() throws FinanceAppException
	{
		try
		{
			InitialContext jndiContext = new InitialContext();
			DataSource ds = (DataSource) jndiContext.lookup("java:/DefaultDS");
			
			Connection conn =  ds.getConnection();
			return conn;
		}
		catch (Exception e)
		{
			throw new FinanceAppException ("getConnection() failed", e);
		}
		
	}

	/**
	 * Checks to see, if the database table is initialized.
	 * 
	 * Implementation tests for the persence of stock quote table by selecting from it.
	 * 
	 * @param conn
	 * @return boolean  returns true if the table is present
	 */
	private boolean isDBPrepared (Connection conn)
	{
		try
		{
			Statement statement = conn.createStatement();
			
	        ResultSet rs = statement.executeQuery(tableExistsSql);
	        
	        rs.close ();
	        
	        statement.close ();
        	return true;
		}
		catch (SQLException e)
		{
			return false;
		}
        
	}
	
	/**
	 * Creates the single table (StockQuote) used in this application.
	 * 
	 * @param conn
	 * @throws SQLException
	 */
	private void createTable (Connection conn) throws SQLException
	{
		
        System.out.println("** Re-creating the table: " + tableName);

        Statement stat = conn.createStatement();

        try 
        {
        	// Try to drop the table
            stat.executeUpdate(dropTableSql);
        } 
        catch (SQLException e) 
        {    
        	// Ignore Exception, because the table may not yet exist
        }
        
        stat.execute(createTableSql);
        
	}

	/**
	 * Initalizes the stock quote table, if it is not initalized.
	 * 
	 * @param conn
	 * @throws SQLException
	 */	
	private void checkAndPrepareDB (Connection conn) throws SQLException
	{
		if (!isDBPrepared (conn))
		{
			createTable (conn);
			System.out.println ("** adding tikers QQQ,DIA");
			addStockQuote(conn, "QQQ", 25.0);
			addStockQuote(conn, "DIA", 80.0);
		}
	}

	private void addStockQuote (Connection conn, String ticker, double price) throws SQLException
	{
        PreparedStatement ps = conn.prepareStatement(insertSql);
        
        ps.setString(1, ticker);
        ps.setDouble(2, price);
        
        ps.execute();
        
        ps.close ();
	}
	
	private double getStockQuote (Connection conn, String ticker) throws SQLException, InvalidTickerException
	{
        PreparedStatement ps = conn.prepareStatement(selectSql);
        
        ps.setString(1, ticker);
        
        ResultSet rs = ps.executeQuery();
        
        double price = 0.0;
        
        if (rs.next())
        {
        	price = rs.getDouble(1);
        }
        else
        {
        	throw new InvalidTickerException ("Quote not available for ticker (" + ticker + ")");
        }
        
        rs.close();
        ps.close();
        
        return price;
	}
	
	
}
